<?php

namespace EventPortal\Tests\Feature\Admin\Team;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Permission\Models\Role;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Domain\User\Models\User;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class UserStoreTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots, SeedRolesAndPermissions;

    /** @test **/
    public function the_create_view_supplies_the_correct_data_to_the_frontend()
    {
        factory(Team::class)->state('snapshot')->create();

        $this->assertMatchesJsonSnapshot(
            $this
                ->actingAsSuperAdmin()
                ->get(route('admin.user.create'))->props()
        );
    }

    /** @test */
    public function a_user_can_be_created()
    {
        $this->withoutExceptionHandling();

        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.user.store'), [
                'first_name' => 'Firstname',
                'last_name' => 'Lastname',
                'email' => 'test@example.com',
                'role_id' => 1,
            ])
            ->assertOk();

        $this->assertDatabaseHas('users', [
            'first_name' => 'Firstname',
            'last_name' => 'Lastname',
        ]);
    }

    /** @test **/
    public function the_first_name_last_name_and_email_are_required()
    {
        $this->markTestSkipped('Need to complete');

        $admin = factory(Admin::class)->create();

        $this
            ->actingAsAdmin($admin)
            ->post(route('admin.user.store'), [
                'first_name' => null,
                'last_name' => null,
            ])
            ->assertSessionHasErrors([
                'first_name',
                'last_name',
            ]);

        $this->assertEquals(1, User::count()); // Admin user already exists
    }

    /** @test */
    public function a_delegate_is_created_when_an_admin_user_is_created(): void
    {
        $this->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.user.store'), [
                'first_name' => 'John',
                'last_name' => 'Smith',
                'email' => 'test_admin@example.com',
                'role_id' => Role::findByName('admin')->id,
            ])
            ->assertOk();

        $this->assertCount(1, Delegate::all());
        Delegate::first()->user->is(Admin::where('email', 'test_admin@example.com')->first());
    }
}
